# SingSurf
A mathematical curve and surface visualiser for singular surface and objects from Singularity Theory.

The program can calculate many of the objects found in Singularity theory:

* Algebraic curves defined by a single polynomial equation in two variables. e.g. a circle 

            x^2 + y^2 - r^2;
* Algebraic surfaces defined by a single polynomial equation in three variables. e.g. a cone

            x^2 + y^2 - z^2;
* Parameterised curves defined by a 3D vector expression in a single variable. e.g. a helix

           [cos(pi t), sin(pi t), t];   
* Parameterised surfaces defined by a 3D vector expression in two variables. e.g. a cross-cap

           [x,x y,y^2];          
* Intersection of surfaces with sets defined by an equation. Can be used to calculate non-polynomial curves.
* Mapping from R^3 to R^3 defined by 3D vector equation in three variables. e.g. a rotation

		   [cos(pi th) x - sin(pi th) y,sin(pi th) x + cos(pi th) y,z];   	   
* Intersections where the equation depends on the definition of another curve or surface. e.g. The profile of a surface

			N . [A,B,C];
			N = diff(S,x) ^^ diff(S,y);	
* Mappings where the equation depends on another surface. For example projection of a curve onto a surface.
* Intersections where the equations depends on a pair of curves. For example the pre-symmetry set of a curve.
* Mapping where the equation depends on a pair of curves. For example the Symmetry set.

## Requirements

The program requires

* Java version 21 or later. You can get the lastest jdk from [Oracle](https://www.oracle.com/java/technologies/downloads/#jdk23-windows) or 
[adoptium](https://adoptium.net/en-GB/)

It also depends on two other packages included in the release:
* JavaView mathematical visualization software from [javaview.de](http://www.javaview.de/) 
* The latest version of the Java Expression Parser [singularsys.com/jep/](https://www.singularsys.com/jep/).

It is a good idea to register your version of JavaView. Registration provides a licence file `jv-lic.lic` which should be copied to the `rsrc` directory, this prevents a notification message appearing.    

## Installation and running

The zip file contains all the necessary files for execution.

There are three different main classes
* `org.singsurf.singsurf.SingSurfPro` the 3D version with all sub-types

* `org.singsurf.singsurf.SingSurf2D` the 2D version optimised for curves in the plane

* `org.singsurf.singsurf.ASurfSimp` a simplified version just with the algebraic surface component

The program should be runable by simple double clicking on the `SingSurf.jar` file. From a command line, to run the 3D version use

        java -jar SingSurf.jar    
To run the 2D version use

        java -cp SingSurf.jar org.singsurf.singsurf.SingSurf2D
        
## User interface

Once the program starts you should see a single windows 

![user interface](doc/main_window.png) 

The left hand panel is where individual curves, surfaces and operators, 
can be controlled.
The middle panel is the main display showing the calculated geometries.
The right hand panel list the different definitions and their outputs.

The File menu allows individual definitions to be loaded and saved. The examples
menu has examples of each type of geometry and operation. 

The button bar has JavaView options. The `JV` button brings up the main JavaView control
window. Other buttons bring up, camera, material and information about a specific geometry.
Other buttons change the view control mode.  Details on these can be found
at [www.javaview.de/doc](http://www.javaview.de/doc/index.html).

The display window menus relate to the SingSurf program. Here you can load a definition, 
or use one of the predefined examples.

The list of the right hand side of the display window shows the different surfaces and operators. 
Clicking on the name will select the item. When selected the controls for 
this item are displayed in control window.

These controls have four tabs: definition, domain, parameters and options. 
In the main definition tab the main definition can be input. The format of this depends of the
type of surface/operator.

#### Definition panel

Contains the expression defining the surface

![Definition panel](doc/def_pan.png)

#### Domain panel 

Controls the range of the input variables, number of steps to take.

![Domain panel](doc/dom_pan.png)

#### Parameter panel

Control any parameters which appears in the definition.

![Parameter panel](doc/param_pan.png)

#### Options panel

Various options controlling the calculation and display of the surface. 

![Options panel](doc/opt_pan.png)

#### Javaview panels

Clicking on the `Display` item at the top of the display window list will bring up the 
JavaView display control.  Here various features of rendering can be controlled. The `Inspector` menu brings up  
further JavaView control panels controlling the Camera, Lights and properties of the Geometry. 

## Types of curve, surface and operators

Basic curves and surfaces:

* **Algebraic curves**: one expression in two variables, either `x^2 + y^2 - r^2`; or with an equality `x^2 + y^2 = r^2`;
* **Algebraic surfaces**: one expression in three variables `x^2+y^2+z^2 = r^2`;
* **Parameterised curve**: a 3D vector expression in one variable `[x,x^2,x^3]`;
* **Parameterised surface**: a 3D vector expression in two variables `[x,y,A x^2+2 B x y + C y^2]`;

Basic operators:

These read in a geometry, apply the operation and produce another geometry as output. Each can have one or more geometries as input.

* **Intersection**: Finds the intersection of a geometry with a surface defined by an implicit equation. For example the intersection with a sphere defined by `x^2+y^2+z^2 = r^2`
* **Clip**: Finds one expression in three variables `x^2+y^2+z^2 = r^2`;
* **Mapping**: a output 3D vector expression in terms of three input variables `[y,-x,z]`;
* **VectorField**:  Defined by vector at each point. [-x,-y,-z]
* **IntegralCurves**: 
* **Extrude**: Creates surfaces like surfaces of revolution with an input curve specifying a cross section.
* **Colour**: Apply a colouring algorithm to a geometry

Advanced operators:

Here the definition of the operator depends on the definition of a curve or surface,
called the **ingredient**. Equation depending on three variables `x`,`y`,`z` and one or two 
ingredients, `S`, `T` representing maps S:(x,y,z)->**R**^3 or S:(x,y,z)->**R**. Expressions can use the variable `S` say `S . [a,b,c];` or it derivatives `diff(S,x)`. For example a profile of a parameterised
surface could be specified as

```
N . [a,b,c];
N = [diff(S,x),diff(S,y),diff(S,z)];
```

Several different advance operators are possible

* **GenIntersection**: intersection depending on one ingredient `S`. Examples include profiles of surfaces, parabolic lines
* **GenClip**: clipping inside a half-space.
* **GenMapping**: mapping depends on one ingredient `S`. Examples include projecting curves in parameter space onto a surface, the Guass map and focal surface.
* **BiInt**: intersection depending on two ingredients `S`, `T`, these can often be the same curve. Example includes the pre-Symmetry Sets.
* **BiMap**: mapping depends on two ingredients `S`, `T`, used to generate the symmetry set from the pre-symmetry set. 
* **GenVecField**: 
* **Ridge**: Ridge type curves depending on a surface and vector field.


   


					               
